import PropTypes from 'prop-types';
import React, { createContext, useRef, useCallback, useMemo, useContext, useReducer, useEffect, useLayoutEffect } from 'react';

const DataLoaderContext = createContext();

const DataLoaderProvider = ({
  children
}) => {
  const cachedData = useRef({});
  const reloads = useRef({});
  const setCachedData = useCallback(compute => {
    if (typeof compute === 'function') {
      cachedData.current = compute(cachedData.current);
    } else {
      cachedData.current = compute;
    }
  }, []);
  const setReloads = useCallback(compute => {
    if (typeof compute === 'function') {
      reloads.current = compute(reloads.current);
    } else {
      reloads.current = compute;
    }
  }, []);
  const addCachedData = useCallback((key, newData) => {
    if (key && typeof key === 'string' && newData) {
      setCachedData(actualCachedData => ({ ...actualCachedData,
        [key]: newData
      }));
    }
  }, [setCachedData]);
  const addReload = useCallback((key, method) => {
    if (key && typeof key === 'string' && method) {
      setReloads(actualReloads => ({ ...actualReloads,
        [key]: method
      }));
    }
  }, [setReloads]);
  const clearReload = useCallback(key => {
    if (key && typeof key === 'string') {
      setReloads(actualReloads => {
        const tmp = actualReloads;
        delete tmp[key];
        return tmp;
      });
    }
  }, [setReloads]);
  const clearAllReloads = useCallback(() => {
    setReloads({});
  }, [setReloads]);
  const clearCachedData = useCallback(key => {
    if (key && typeof key === 'string') {
      setCachedData(actualCachedData => {
        const tmp = actualCachedData;
        delete tmp[key];
        return tmp;
      });
    }
  }, [setCachedData]);
  const clearAllCachedData = useCallback(() => {
    setCachedData({});
  }, [setCachedData]);
  const reload = useCallback(async key => {
    if (key && typeof key === 'string') {
      await (reloads.current[key] && reloads.current[key]());
    }
  }, []);
  const reloadAll = useCallback(async () => {
    await Promise.all(Object.values(reloads.current).map(reloadFn => reloadFn()));
  }, []);
  const getCachedData = useCallback(key => {
    if (key) {
      return cachedData.current[key] || undefined;
    }

    return cachedData.current;
  }, []);
  const getReloads = useCallback(key => {
    if (key) {
      return reloads.current[key] || undefined;
    }

    return reloads.current;
  }, []);
  const value = useMemo(() => ({
    addCachedData,
    clearCachedData,
    getCachedData,
    reload,
    getReloads,
    addReload,
    clearReload,
    clearAllCachedData,
    clearAllReloads,
    reloadAll
  }), [addCachedData, clearReload, clearCachedData, getCachedData, getReloads, addReload, reload, clearAllCachedData, clearAllReloads, reloadAll]);
  return /*#__PURE__*/React.createElement(DataLoaderContext.Provider, {
    value: value
  }, children);
};

DataLoaderProvider.propTypes = {
  children: PropTypes.node.isRequired
};
const useDataLoaderContext = () => useContext(DataLoaderContext);

const StatusEnum = {
  LOADING: 'loading',
  SUCCESS: 'success',
  ERROR: 'error',
  IDLE: 'idle'
};
const ActionEnum = {
  RESET: 'RESET',
  ON_LOADING: 'ON_LOADING',
  ON_SUCCESS: 'ON_SUCCESS',
  ON_UPDATE_DATA: 'ON_UPDATE_DATA',
  ON_ERROR: 'ON_ERROR'
};

/* eslint-disable consistent-return */
var reducer = ((state, action) => {
  switch (action.type) {
    case ActionEnum.ON_LOADING:
      return { ...state,
        error: undefined,
        status: StatusEnum.LOADING
      };

    case ActionEnum.ON_SUCCESS:
      return { ...state,
        error: undefined,
        data: action.data,
        status: StatusEnum.SUCCESS
      };

    case ActionEnum.ON_UPDATE_DATA:
      return { ...state,
        data: action.data
      };

    case ActionEnum.RESET:
      return {
        status: StatusEnum.IDLE,
        data: action.data,
        error: undefined
      };

    case ActionEnum.ON_ERROR:
      return { ...state,
        error: action.error,
        status: StatusEnum.ERROR
      };
  }
});

const Actions = {
  createReset: ({
    data
  }) => ({
    type: ActionEnum.RESET,
    data
  }),
  createOnLoading: () => ({
    type: ActionEnum.ON_LOADING
  }),
  createOnSuccess: data => ({
    type: ActionEnum.ON_SUCCESS,
    data
  }),
  createOnUpdateData: data => ({
    type: ActionEnum.ON_UPDATE_DATA,
    data
  }),
  createOnError: error => ({
    type: ActionEnum.ON_ERROR,
    error
  })
};

const useDataLoader = (key, method, {
  onSuccess,
  onError,
  initialData,
  pollingInterval,
  enabled = true,
  reloadOnKeyChange = true,
  keepPreviousData = true
} = {}) => {
  const {
    addCachedData,
    addReload,
    clearReload,
    getCachedData
  } = useDataLoaderContext();
  const [{
    status,
    data,
    error
  }, dispatch] = useReducer(reducer, {
    status: StatusEnum.IDLE,
    data: initialData,
    error: undefined
  });
  const previousDataRef = useRef();
  const keyRef = useRef();
  const methodRef = useRef(method);
  const onSuccessRef = useRef(onSuccess);
  const onErrorRef = useRef(onError);
  const isLoading = useMemo(() => status === StatusEnum.LOADING, [status]);
  const isIdle = useMemo(() => status === StatusEnum.IDLE, [status]);
  const isSuccess = useMemo(() => status === StatusEnum.SUCCESS, [status]);
  const isError = useMemo(() => status === StatusEnum.ERROR, [status]);
  const isPolling = useMemo(() => enabled && pollingInterval && (isSuccess || isLoading), [isSuccess, isLoading, enabled, pollingInterval]);
  const handleRequest = useRef(async (cacheKey, args) => {
    const cachedData = getCachedData(cacheKey);

    if (cacheKey && !data && cachedData) {
      dispatch(Actions.createOnUpdateData(cachedData));
    }

    try {
      var _methodRef$current, _onSuccessRef$current;

      dispatch(Actions.createOnLoading());
      const result = await ((_methodRef$current = methodRef.current) === null || _methodRef$current === void 0 ? void 0 : _methodRef$current.call(methodRef, args));
      if (result && cacheKey) addCachedData(cacheKey, result);

      if (keyRef.current && cacheKey && cacheKey !== keyRef.current) {
        return;
      }

      dispatch(Actions.createOnSuccess(result));
      await ((_onSuccessRef$current = onSuccessRef.current) === null || _onSuccessRef$current === void 0 ? void 0 : _onSuccessRef$current.call(onSuccessRef, result));
    } catch (err) {
      var _onErrorRef$current;

      dispatch(Actions.createOnError(err));
      await ((_onErrorRef$current = onErrorRef.current) === null || _onErrorRef$current === void 0 ? void 0 : _onErrorRef$current.call(onErrorRef, err));
    }
  });
  useEffect(() => {
    let handler;

    if (enabled) {
      if (reloadOnKeyChange && key !== keyRef.current && status !== StatusEnum.IDLE) {
        keyRef.current = key;
        dispatch(Actions.createReset({
          data: initialData
        }));
      } else {
        if (status === StatusEnum.IDLE) {
          keyRef.current = key;
          handleRequest.current(key);
        }

        if (pollingInterval && status === StatusEnum.SUCCESS) {
          handler = setTimeout(() => handleRequest.current(key), pollingInterval);
        }

        if (key && typeof key === 'string') {
          addReload(key, reloadArgs => handleRequest.current(key, reloadArgs));
        }
      }
    }

    return () => {
      if (key && typeof key === 'string') {
        clearReload(key);
      }

      if (handler) {
        clearTimeout(handler);
      }
    }; // Why can't put empty array for componentDidMount, componentWillUnmount ??? No array act like componentDidMount and componentDidUpdate
  }, [enabled, pollingInterval, key, clearReload, addReload, status, reloadOnKeyChange, initialData]);
  useLayoutEffect(() => {
    methodRef.current = method;
  }, [method]);
  useLayoutEffect(() => {
    if (keepPreviousData && data && previousDataRef.current !== data) {
      previousDataRef.current = data;
    }
  }, [keepPreviousData, data]);
  return {
    isLoading,
    isIdle,
    isSuccess,
    isError,
    isPolling,
    previousData: previousDataRef.current,
    data,
    error,
    reload: args => handleRequest.current(key, args)
  };
};

export { DataLoaderProvider, useDataLoader, useDataLoaderContext };
